@@ -4,7 +4,7 @@
 # in device item
 #
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 setup_root_helper
 
@@ -14,7 +14,7 @@ _mktemp_local "$file" 1g
 
 dev=$(run_check_stdout $SUDO_HELPER losetup --find --show "$file")
 
-run_check "$TOP/mkfs.btrfs" -f "$dev"
+run_check $SUDO_HELPER "$TOP/mkfs.btrfs" -f "$dev"
 
 # The original device size from prepare_loopdevs is 2G.
 # Since the fs is empty, shrinking it to 996m will not cause any
@@ -26,5 +26,5 @@ dev=$(run_check_stdout $SUDO_HELPER losetup --find --show "$file")
 run_mustfail "btrfs check should detect errors in device size" \
 	"$TOP/btrfs" check "$dev"
 
-losetup -d "$dev"
+$SUDO_HELPER losetup -d "$dev"
 rm -- "$file"
